///////////////////////////////////////////
// Processingによる二分法の対話的プログラム
//                      2012/04/28 T.Shirai
//             shirai＠mech.suzuka-ct.ac.jp
///////////////////////////////////////////

// 動作に関わるオプション．ドキュメントを読んで理解したら自由に設定して構いません．
void Options() {
//   crowbar.enableLogOutput();           // default.txtにログを記録する
}

// パラメータ入力前に行っておきたい処理（setup()の最初に自動実行）
void initializeMain() {
  // 以下の行は削除してはいけない．二つの引数の値を変更するとウィンドウサイズが変わる
  crowbar = new crowbarClass(60, 30);
}

///////////////////////////////////////////////////////////////////////////////////
// ここから下のコードはユーザが積極的に記述して構わない
///////////////////////////////////////////////////////////////////////////////////
// システム初期設定
void Setup()
{
  // プログラムコメント（パラメータ入力前に画面表示されます．複数宣言可，省略可）
  crowbar.programComment("二分法のプログラム");
  crowbar.programComment("区間aは区間bよりも小さな値（例： a = 1, b = 2）");

  // 以下に，入力が必要なパラメータを宣言
  crowbar.define("区間ａ");
  crowbar.define("区間ｂ");
}

// メインルーチン
// void Main() の関数名を変えてはいけない．
void Main() {
  float err;
  float a, b, c, tmp;
  int   loopcount;
  // 許容誤差
  err = 0.000001;
  // 変数の代入
  a = crowbar.getFloat();
  b = crowbar.getFloat();
  // 範囲のチェックその１
  if (a > b) {
    // swap
    tmp = a; a = b; b = tmp;
  }
  // 範囲のチェックその２
  if (sgn(f(a)) == sgn(f(b))) {
    crowbar.writeln("Error: 値の範囲が正しくありません．");
    crowbar.writeln("f(a) = " + nf(f(a), 1, 4) + " f(b) = " + nf(f(b), 1, 3));
    return;
  }
  // 繰り返し計算
  loopcount = 0;
  displayBSM(loopcount, a, b);
  while (abs(b - a) >= err) {
    c = (a + b) / 2.0;
    if (sgn(f(a)) == sgn(f(c))) a = c; else b = c;
    loopcount++;
    displayBSM(loopcount, a, b);
  }
  // 最終結果
  crowbar.newline();
  crowbar.writeln(str(loopcount) + "回で収束しました．");
  crowbar.writeln("結果は" + str((a + b) / 2.0) + "です．");
}

// 問題の方程式
float f(float x) {
  return (pow(x,3) - 3*pow(x,2) + 9*x -8);
}

// 区間aと区間bの値を表示する関数
void displayBSM(int i, float a, float b) {
  crowbar.write("[" + nf(i, 3) + "] ");
  crowbar.write("区間A : xa = " + nf(a, 1, 4) + " f(xa) = " + nf(f(a), 1, 4));
  crowbar.write(" / ");
  crowbar.write("区間B : xb = " + nf(b, 1, 4) + " f(xb) = " + nf(f(b), 1, 4));
  crowbar.newline();
}

